@@ -235,13 +235,13 @@ static void destroy_read_heap (void) /* {{{ */
 
 	while (42)
 	{
-		callback_func_t *cf;
+		read_func_t *rf;
 
-		cf = c_heap_get_root (read_heap);
-		if (cf == NULL)
+		rf = c_heap_get_root (read_heap);
+		if (rf == NULL)
 			break;
-
-		destroy_callback (cf);
+		sfree (rf->rf_name);
+		destroy_callback ((callback_func_t *) rf);
 	}
 
 	c_heap_destroy (read_heap);
@@ -974,7 +974,7 @@ static int plugin_mark_loaded (char const *name)
 	return (status);
 }
 
-static void plugin_free_loaded ()
+static void plugin_free_loaded (void)
 {
 	void *key;
 	void *value;
@@ -1237,8 +1237,10 @@ int plugin_register_read (const char *name,
 	rf->rf_interval = plugin_get_interval ();
 
 	status = plugin_insert_read (rf);
-	if (status != 0)
+	if (status != 0) {
+		sfree (rf->rf_name);
 		sfree (rf);
+	}
 
 	return (status);
 } /* int plugin_register_read */
@@ -1285,8 +1287,10 @@ int plugin_register_complex_read (const char *group, const char *name,
 	rf->rf_ctx = plugin_get_ctx ();
 
 	status = plugin_insert_read (rf);
-	if (status != 0)
+	if (status != 0) {
+		sfree (rf->rf_name);
 		sfree (rf);
+	}
 
 	return (status);
 } /* int plugin_register_complex_read */
@@ -1563,7 +1567,6 @@ int plugin_unregister_notification (const char *name)
 void plugin_init_all (void)
 {
 	char const *chain_name;
-	long write_threads_num;
 	llentry_t *le;
 	int status;
 
@@ -1609,8 +1612,6 @@ void plugin_init_all (void)
 		write_threads_num = 5;
 	}
 
-	start_write_threads ((size_t) write_threads_num);
-
 	if ((list_init == NULL) && (read_heap == NULL))
 		return;
 
@@ -1646,6 +1647,8 @@ void plugin_init_all (void)
 		le = le->next;
 	}
 
+	start_write_threads ((size_t) write_threads_num);
+
 	max_read_interval = global_option_get_time ("MaxReadInterval",
 			DEFAULT_MAX_READ_INTERVAL);
 
@@ -1720,6 +1723,7 @@ int plugin_read_all_once (void)
 			return_status = -1;
 		}
 
+		sfree (rf->rf_name);
 		destroy_callback ((void *) rf);
 	}
 
@@ -2267,7 +2271,7 @@ int plugin_dispatch_multivalue (value_list_t const *template, /* {{{ */
 		case DS_TYPE_GAUGE:
 			vl->values[0].gauge = va_arg (ap, gauge_t);
 			if (store_percentage)
-				vl->values[0].gauge *= 100.0 / sum;
+				vl->values[0].gauge *= sum ? (100.0 / sum) : 0;
 			break;
 		case DS_TYPE_ABSOLUTE:
 			vl->values[0].absolute = va_arg (ap, absolute_t);
