@@ -166,16 +166,16 @@ struct resp_pkt
 /* l_fp to double */
 #define M_LFPTOD(r_i, r_uf, d) \
 	do { \
-		register int32_t  i; \
-		register uint32_t f; \
+		register int32_t  ri; \
+		register uint32_t rf; \
 		\
-		i = (r_i); \
-		f = (r_uf); \
-		if (i < 0) { \
-			M_NEG(i, f); \
-			(d) = -((double) i + ((double) f) / 4294967296.0); \
+		ri = (r_i); \
+		rf = (r_uf); \
+		if (ri < 0) { \
+			M_NEG(ri, rf); \
+			(d) = -((double) ri + ((double) rf) / 4294967296.0); \
 		} else { \
-			(d) = (double) i + ((double) f) / 4294967296.0; \
+			(d) = (double) ri + ((double) rf) / 4294967296.0; \
 		} \
 	} while (0)
 
@@ -310,7 +310,7 @@ static int ntpd_config (const char *key, const char *value)
 	return (0);
 }
 
-static void ntpd_submit (char *type, char *type_inst, double value)
+static void ntpd_submit (char *type, char *type_inst, gauge_t value)
 {
 	value_t values[1];
 	value_list_t vl = VALUE_LIST_INIT;
@@ -332,7 +332,7 @@ static void ntpd_submit (char *type, char *type_inst, double value)
  * sets the LSB based on whether the peer was reachable. If the LSB is zero,
  * the values are out of date. */
 static void ntpd_submit_reach (char *type, char *type_inst, uint8_t reach,
-		double value)
+		gauge_t value)
 {
 	if (!(reach & 1))
 		value = NAN;
@@ -670,7 +670,6 @@ static int ntpd_receive_response (int *res_items, int *res_size,
 				(items_num + pkt_item_num) * res_item_size);
 		if (items == NULL)
 		{
-			items = *res_data;
 			ERROR ("ntpd plugin: realloc failed.");
 			continue;
 		}
@@ -906,9 +905,19 @@ static int ntpd_read (void)
 	int                       ps_num;
 	int                       ps_size;
 
+	gauge_t offset_loop;
+	gauge_t freq_loop;
+	gauge_t offset_error;
+
 	int status;
 	int i;
 
+	/* On Linux, if the STA_NANO bit is set in ik->status, then ik->offset
+	 * is is nanoseconds, otherwise it's microseconds.
+	 * TODO(octo): STA_NANO is defined in the Linux specific <sys/timex.h> header. */
+	double scale_loop  = 1e-6;
+	double scale_error = 1e-6;
+
 	ik      = NULL;
 	ik_num  = 0;
 	ik_size = 0;
@@ -931,18 +940,19 @@ static int ntpd_read (void)
 	}
 
 	/* kerninfo -> estimated error */
+	offset_loop  = scale_loop * ((gauge_t) ntohl (ik->offset));
+	freq_loop    = ntpd_read_fp (ik->freq);
+	offset_error = scale_error * ((gauge_t) ntohl (ik->esterror));
 
 	DEBUG ("info_kernel:\n"
-			"  pll offset    = %.8f\n"
-			"  pll frequency = %.8f\n" /* drift compensation */
-			"  est error     = %.8f\n",
-			ntpd_read_fp (ik->offset),
-			ntpd_read_fp (ik->freq),
-			ntpd_read_fp (ik->esterror));
+			"  pll offset    = %.8g\n"
+			"  pll frequency = %.8g\n" /* drift compensation */
+			"  est error     = %.8g\n",
+			offset_loop, freq_loop, offset_error);
 
-	ntpd_submit ("frequency_offset", "loop",  ntpd_read_fp (ik->freq));
-	ntpd_submit ("time_offset",      "loop",  ntpd_read_fp (ik->offset));
-	ntpd_submit ("time_offset",      "error", ntpd_read_fp (ik->esterror));
+	ntpd_submit ("frequency_offset", "loop",  freq_loop);
+	ntpd_submit ("time_offset",      "loop",  offset_loop);
+	ntpd_submit ("time_offset",      "error", offset_error);
 
 	free (ik);
 	ik = NULL;
