@@ -23,7 +23,6 @@
 #include "plugin.h"
 #include "common.h"
 #include "utils_cache.h"
-#include "utils_parse_option.h"
 
 #include <tcrdb.h>
 
@@ -79,7 +78,7 @@ static int tt_config (const char *key, const char *value)
 	return (0);
 }
 
-static void printerr()
+static void printerr (void)
 {
 	int ecode = tcrdbecode(rdb);
 	ERROR ("tokyotyrant plugin: error: %d, %s",
